@@ -1597,6 +1597,7 @@ generate_cache_handler (LDAPOp *op, LDAPMessage *res)
 		GList *l;
 
 		e_file_cache_clean (E_FILE_CACHE (bl->priv->cache));
+		e_file_cache_freeze_changes (E_FILE_CACHE (bl->priv->cache));
 		
 		for (l = contact_list_op->contacts; l; l = g_list_next (l)) {
 			EContact *contact = l->data;
@@ -1604,6 +1605,7 @@ generate_cache_handler (LDAPOp *op, LDAPMessage *res)
 		}
 
 		e_book_backend_cache_set_populated (bl->priv->cache);
+		e_file_cache_thaw_changes (E_FILE_CACHE (bl->priv->cache));
 		ldap_op_finished (op);
 	}
 }
@@ -1710,6 +1712,7 @@ set_mode (EBookBackend *backend, int mode)
 {
 	EBookBackendGAL *be = E_BOOK_BACKEND_GAL (backend);
 	EBookBackendGALPrivate *bepriv;
+	ExchangeAccountResult result;
 
 	bepriv = be->priv;
 
@@ -1731,7 +1734,7 @@ set_mode (EBookBackend *backend, int mode)
 			}
 			}
 		} else if (mode == GNOME_Evolution_Addressbook_MODE_REMOTE) {
-			if (exchange_account_connect (bepriv->account)) {
+			if (exchange_account_connect (bepriv->account, NULL, &result)) {
 				e_book_backend_set_is_writable (backend, FALSE);
 				e_book_backend_notify_writable (backend, FALSE);
 				e_book_backend_notify_connection_status (backend, TRUE);
