@@ -158,6 +158,28 @@ camel_exchange_store_get_type (void)
 	return camel_exchange_store_type;
 }
 
+/* Use this to ensure that the camel session is online and we are connected
+   too. Also returns the current status of the store */
+gboolean
+camel_exchange_store_connected (CamelExchangeStore *store, CamelException *ex)
+{
+	CamelService *service;
+	CamelSession *session;
+
+	g_return_val_if_fail (CAMEL_IS_EXCHANGE_STORE (store), FALSE);
+
+	service = CAMEL_SERVICE (store);
+	session = service->session;
+
+	if (service->status != CAMEL_SERVICE_CONNECTED &&
+	    camel_session_is_online (session) &&
+	    !camel_service_connect (service, ex)) {
+		return FALSE;
+	}
+
+	return CAMEL_OFFLINE_STORE (store)->state != CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL;
+}
+
 /* This has been now removed from evolution/e-util. So implemented this here.
  * Also note that this is similar to the call in e2k-path.c. The name of the 
  * function has been changed to avoid any conflicts.
@@ -312,11 +334,31 @@ e_filename_make_safe (gchar *string)
 	}
 }
 
+static void
+camel_exchange_get_password (CamelService *service, CamelException *ex)
+{
+	CamelSession *session = camel_service_get_session (service);
+	char *prompt, *account_name;
+
+	if (!service->url->passwd) {
+		/* This is a hack to use the same string being used in 
+		   exchange-account.c:get_password */
+		account_name = g_strdup_printf ("%s@%s", service->url->user, service->url->host);
+		prompt = g_strdup_printf (_("%sEnter password for %s"), "", account_name);
+
+		service->url->passwd = camel_session_get_password (session, 
+					service, "Exchange", prompt, "password", CAMEL_SESSION_PASSWORD_SECRET, ex);
+		g_free (account_name);
+		g_free (prompt);
+	}
+}
+
 static gboolean
 exchange_connect (CamelService *service, CamelException *ex)
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (service);
 	char *real_user, *socket_path;
+	guint32 connect_status;
 
 	if (exch->stub)
 		return TRUE;
@@ -336,9 +378,12 @@ exchange_connect (CamelService *service, CamelException *ex)
 	if (!exch->stub)
 		return FALSE;
 	
+	camel_exchange_get_password (service, ex);
 	/* Initialize the stub connection */
 	if (!camel_stub_send (exch->stub, NULL, CAMEL_STUB_CMD_CONNECT,
+			      CAMEL_STUB_ARG_STRING, g_strdup (service->url->passwd),
 			      CAMEL_STUB_ARG_RETURN,
+			      CAMEL_STUB_ARG_UINT32, &connect_status,
 			      CAMEL_STUB_ARG_END)) {
 		/* The user cancelled the connection attempt. */
 		camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
@@ -359,8 +404,11 @@ exchange_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 	return TRUE;
 }
 
-#define RETURN_VAL_IF_NOT_CONNECTED(service, ex, val)\
-	if (!exchange_connect((CamelService *)service, ex))\
+/* Even if we are disconnected, we need to exchange_connect
+   to get the offline data */
+#define RETURN_VAL_IF_NOT_CONNECTED(store, ex, val)\
+	if (!camel_exchange_store_connected(store, ex) && \
+	    !exchange_connect (CAMEL_SERVICE (store), ex)) \
 		return val; 
 
 static CamelFolder *
@@ -371,11 +419,11 @@ exchange_get_folder (CamelStore *store, const char *folder_name,
 	CamelFolder *folder;
 	char *folder_dir;
 	
-	RETURN_VAL_IF_NOT_CONNECTED(store, ex, NULL);
+	RETURN_VAL_IF_NOT_CONNECTED (exch, ex, NULL);
 
 	folder_dir = exchange_path_to_physical (exch->storage_path, folder_name);
 	
-	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+	if (!camel_exchange_store_connected (exch, ex)) {
 		if (!folder_dir || access (folder_dir, F_OK) != 0) {
 			g_free (folder_dir);
 			camel_exception_setv (ex, CAMEL_EXCEPTION_STORE_NO_FOLDER,
@@ -430,7 +478,7 @@ exchange_folder_subscribed (CamelStore *store, const char *folder_name)
 	}
 
 	if (!camel_stub_send (exch->stub, NULL, CAMEL_STUB_CMD_IS_SUBSCRIBED_FOLDER,
-			      CAMEL_STUB_ARG_STRING, folder_name,
+			      CAMEL_STUB_ARG_FOLDER, folder_name,
 			      CAMEL_STUB_ARG_RETURN,
 			      CAMEL_STUB_ARG_UINT32, &is_subscribed,
 			      CAMEL_STUB_ARG_END)) {
@@ -447,13 +495,13 @@ exchange_subscribe_folder (CamelStore *store, const char *folder_name,
 	d(printf ("subscribe folder : %s\n", folder_name));
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 	
-	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+	if (!camel_exchange_store_connected (exch, ex)) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot subscribe folder in offline mode."));
 		return;
 	}
 
 	camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_SUBSCRIBE_FOLDER,
-			      CAMEL_STUB_ARG_STRING, folder_name,
+			      CAMEL_STUB_ARG_FOLDER, folder_name,
 			      CAMEL_STUB_ARG_END);
 }
 
@@ -464,13 +512,13 @@ exchange_unsubscribe_folder (CamelStore *store, const char *folder_name,
 	d(printf ("unsubscribe folder : %s\n", folder_name));
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 	
-	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+	if (!camel_exchange_store_connected (exch, ex)) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot unsubscribe folder in offline mode."));
 		return;
 	}
 
 	camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_UNSUBSCRIBE_FOLDER,
-			      CAMEL_STUB_ARG_STRING, folder_name,
+			      CAMEL_STUB_ARG_FOLDER, folder_name,
 			      CAMEL_STUB_ARG_END);
 }
 
@@ -480,7 +528,7 @@ get_trash (CamelStore *store, CamelException *ex)
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 	
-	RETURN_VAL_IF_NOT_CONNECTED(store, ex, NULL);
+	RETURN_VAL_IF_NOT_CONNECTED (exch, ex, NULL);
 
 	if (!exch->trash_name) {
 		if (!camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_GET_TRASH_NAME,
@@ -579,7 +627,7 @@ exchange_get_folder_info (CamelStore *store, const char *top, guint32 flags, Cam
 	 * each time auto-send/recv runs.
 	 */
 
-	RETURN_VAL_IF_NOT_CONNECTED(store, ex, NULL);
+	RETURN_VAL_IF_NOT_CONNECTED (exch, ex, NULL);
 
 	if (camel_stub_marshal_eof (exch->stub->cmd))
 		return NULL;
@@ -623,6 +671,7 @@ exchange_get_folder_info (CamelStore *store, const char *top, guint32 flags, Cam
 	g_array_free (folder_flags, TRUE);
 
 	info = camel_folder_info_build (folders, top, '/', TRUE);
+	
 	if (info)
 		info = postprocess_tree (info);
 	g_ptr_array_free (folders, TRUE);
@@ -639,7 +688,7 @@ exchange_create_folder (CamelStore *store, const char *parent_name,
 	guint32 unread_count, flags;
 	CamelFolderInfo *info;
 	
-	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+	if (!camel_exchange_store_connected (exch, ex)) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot create folder in offline mode."));
 		return NULL;
 	}
@@ -666,7 +715,7 @@ exchange_delete_folder (CamelStore *store, const char *folder_name,
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 
-	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+	if (!camel_exchange_store_connected (exch, ex)) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot delete folder in offline mode."));
 		return;
 	}
@@ -682,7 +731,7 @@ exchange_rename_folder (CamelStore *store, const char *old_name,
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 
-	if (((CamelOfflineStore *) store)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+	if (!camel_exchange_store_connected (exch, ex)) {
 		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot rename folder in offline mode."));
 		return;
 	}
@@ -793,6 +842,31 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		break;
 	}
 
+	case CAMEL_STUB_RETVAL_CHANGED_FLAGS_EX:
+	{
+		CamelExchangeFolder *folder;
+		char *folder_name, *uid;
+		guint32 flags;
+		guint32 mask;
+
+		if (camel_stub_marshal_decode_folder (stub->status, &folder_name) == -1 ||
+		    camel_stub_marshal_decode_string (stub->status, &uid) == -1 ||
+		    camel_stub_marshal_decode_uint32 (stub->status, &flags) == -1 ||
+		    camel_stub_marshal_decode_uint32 (stub->status, &mask) == -1)
+			break;
+
+		g_mutex_lock (exch->folders_lock);
+		folder = g_hash_table_lookup (exch->folders, folder_name);
+		g_mutex_unlock (exch->folders_lock);
+		if (folder)
+			camel_exchange_folder_update_message_flags_ex (folder, uid,
+								       flags, mask);
+
+		g_free (folder_name);
+		g_free (uid);
+		break;
+	}
+
 	case CAMEL_STUB_RETVAL_CHANGED_TAG:
 	{
 		CamelExchangeFolder *folder;
@@ -865,7 +939,7 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		info = make_folder_info (exch, name, uri, -1, 0);
 		info->flags |= CAMEL_FOLDER_NOCHILDREN;
 		camel_object_trigger_event (CAMEL_OBJECT (exch),
-					    "folder_created", info);
+					    "folder_subscribed", info);
 		camel_folder_info_free (info);
 		break;
 	}
@@ -873,15 +947,25 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 	case CAMEL_STUB_RETVAL_FOLDER_DELETED:
 	{
 		CamelFolderInfo *info;
+		CamelFolder *folder;
 		char *name, *uri;
 
 		if (camel_stub_marshal_decode_string (stub->status, &name) == -1 ||
 		    camel_stub_marshal_decode_string (stub->status, &uri) == -1)
 			break;
-
+		
 		info = make_folder_info (exch, name, uri, -1, 0);
+
+		g_mutex_lock (exch->folders_lock);
+		folder = g_hash_table_lookup (exch->folders, info->full_name);
+		if (folder) {
+			g_hash_table_remove (exch->folders, info->full_name);
+			camel_object_unref (CAMEL_OBJECT (folder));
+		}
+		g_mutex_unlock (exch->folders_lock);
+
 		camel_object_trigger_event (CAMEL_OBJECT (exch),
-					    "folder_deleted", info);
+					    "folder_unsubscribed", info);
 		camel_folder_info_free (info);
 		break;
 	}
@@ -889,6 +973,7 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 	case CAMEL_STUB_RETVAL_FOLDER_RENAMED:
 	{
 		char *new_name, *new_uri;
+		CamelFolder *folder;
 		CamelRenameInfo reninfo;
 
 		if (camel_stub_marshal_decode_folder (stub->status, &reninfo.old_base) == -1 ||
@@ -897,6 +982,15 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 			break;
 
 		reninfo.new = make_folder_info (exch, new_name, new_uri, -1, 0);
+		
+		g_mutex_lock (exch->folders_lock);
+		folder = g_hash_table_lookup (exch->folders, reninfo.old_base);
+		if (folder) {
+			g_hash_table_remove (exch->folders, reninfo.old_base);
+			camel_object_unref (CAMEL_OBJECT (folder));
+		}
+		g_mutex_unlock (exch->folders_lock);
+		
 		camel_object_trigger_event (CAMEL_OBJECT (exch),
 					    "folder_renamed", &reninfo);
 		camel_folder_info_free (reninfo.new);
