@@ -29,21 +29,16 @@
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-main.h>
 
-#include "e-storage-set.h"
-#include "e-storage-set-view.h"
-
-#include "exchange-account.h"
+#include <exchange-account.h>
+#include <exchange-constants.h>
 #include "exchange-config-listener.h"
-#include "exchange-hierarchy-gal.h"
 #include "exchange-oof.h"
-#include "exchange-storage.h"
-#include "e-folder-exchange.h"
+#include <e-folder-exchange.h>
+#include <e-shell-marshal.h>
 
 #include "mail-stub-listener.h"
 #include "mail-stub-exchange.h"
 
-#include "xc-backend-view.h"
-
 #include "exchange-migrate.h" 
 
 #define d(x)
@@ -51,17 +46,30 @@
 #define PARENT_TYPE bonobo_object_get_type ()
 static BonoboObjectClass *parent_class = NULL;
 static gboolean idle_do_interactive (gpointer user_data);
+static void exchange_component_update_accounts (ExchangeComponent *component,
+							gboolean status);
+static void ex_migrate_esources (ExchangeComponent *component,
+				  const CORBA_short major, 
+				  const CORBA_short minor, 
+				  const CORBA_short revision);
+
+static guint linestatus_signal_id;
 
 struct ExchangeComponentPrivate {
 	GdkNativeWindow xid;
 
-	EFolderTypeRegistry *folder_type_registry;
-
 	ExchangeConfigListener *config_listener;
-	ExchangeOfflineListener *offline_listener;
+
+	EDataCalFactory *cal_factory;
+	EDataBookFactory *book_factory;
+
+	gboolean linestatus;
+
 	GSList *accounts;
 
 	GSList *views;
+
+	GNOME_Evolution_Listener evo_listener;
 };
 
 typedef struct {
@@ -83,11 +91,6 @@ dispose (GObject *object)
 	ExchangeComponentPrivate *priv = EXCHANGE_COMPONENT (object)->priv;
 	GSList *p;
 
-	if (priv->folder_type_registry) {
-		g_object_unref (priv->folder_type_registry);
-		priv->folder_type_registry = NULL;
-	}
-
 	if (priv->config_listener) {
 		g_object_unref (priv->config_listener);
 		priv->config_listener = NULL;
@@ -107,9 +110,26 @@ dispose (GObject *object)
 		priv->views = NULL;
 	}
 
-	if (priv->offline_listener) {
-		g_object_unref (priv->offline_listener);
-		priv->offline_listener = NULL;
+	if (priv->cal_factory) {
+		g_object_unref (priv->cal_factory);
+		priv->cal_factory = NULL;
+	}
+
+	if (priv->book_factory) {
+		g_object_unref (priv->book_factory);
+		priv->book_factory = NULL;
+	}
+
+	if (priv->evo_listener) {
+		CORBA_Environment ev;
+
+		CORBA_exception_init (&ev);
+		GNOME_Evolution_Listener_complete (priv->evo_listener, &ev);
+		CORBA_exception_free (&ev);
+		CORBA_Object_release (priv->evo_listener, &ev);
+		CORBA_exception_free (&ev);
+		
+		priv->evo_listener = NULL;
 	}
 
 	G_OBJECT_CLASS (parent_class)->dispose (object);
@@ -128,29 +148,9 @@ impl_createControls (PortableServer_Servant servant,
 		     Bonobo_Control *statusbar_control,
 		     CORBA_Environment *ev)
 {
-	ExchangeComponent *component = EXCHANGE_COMPONENT (bonobo_object_from_servant (servant));
-	ExchangeComponentPrivate *priv = component->priv;
-	XCBackendView *view;
-	BonoboControl *control = NULL;
-
 	d(printf("createControls...\n"));
 
-	view = xc_backend_view_new (priv->config_listener,
-				    priv->folder_type_registry);
-	if (view)
-		priv->views = g_slist_append (priv->views, view);
-
-	control = xc_backend_view_get_sidebar (view);
-	*sidebar_control =
-		CORBA_Object_duplicate (BONOBO_OBJREF (control), ev);
-
-	control = xc_backend_view_get_statusbar (view);
-	*statusbar_control =
-		CORBA_Object_duplicate (BONOBO_OBJREF (control), ev);
-
-	control = xc_backend_view_get_view (view);
-	*view_control =
-		CORBA_Object_duplicate (BONOBO_OBJREF (control), ev);
+	*sidebar_control = *view_control = *statusbar_control = NULL;
 }
 
 static void
@@ -176,6 +176,8 @@ impl_upgradeFromVersion (PortableServer_Servant servant,
 
 		exchange_migrate(major, minor, revision, 
 				 base_directory, account->account_filename);
+				 	
+		ex_migrate_esources (component, major, minor, revision);			
 	}
 }
 
@@ -242,6 +244,72 @@ impl_interactive (PortableServer_Servant servant,
 		priv->xid = 0;
 }
 
+static void
+impl_setLineStatus (PortableServer_Servant servant,
+		    const CORBA_boolean status,
+		    const GNOME_Evolution_Listener listener,
+		    CORBA_Environment *ev)
+{
+	ExchangeComponent *component = EXCHANGE_COMPONENT (bonobo_object_from_servant (servant));
+	ExchangeComponentPrivate *priv = component->priv;
+
+	priv->linestatus = status;
+	
+	if (priv->cal_factory) {
+		e_data_cal_factory_set_backend_mode (priv->cal_factory,
+						     status ? ONLINE_MODE : OFFLINE_MODE);
+	}
+
+	if (priv->book_factory) {
+		e_data_book_factory_set_backend_mode (priv->book_factory,
+						      status ? ONLINE_MODE : OFFLINE_MODE);
+	}
+
+	if (priv->evo_listener == NULL) {
+		priv->evo_listener = CORBA_Object_duplicate (listener, ev);
+		if (ev->_major == CORBA_NO_EXCEPTION) {
+			exchange_component_update_accounts (component, status);
+			g_signal_emit (component, linestatus_signal_id, 0,
+				       status ? ONLINE_MODE : OFFLINE_MODE);
+			return;
+		} else {
+			CORBA_exception_free (ev);
+		}
+	}
+
+	GNOME_Evolution_Listener_complete (listener, ev);
+}
+
+static void
+ex_migrate_esources (ExchangeComponent *component,
+		  const CORBA_short major, 
+		  const CORBA_short minor, 
+		  const CORBA_short revision)
+{
+
+	if ((major == 1) || ((major == 2) && (minor <= 3)))
+	{
+		exchange_config_listener_migrate_esources (component->priv->config_listener);
+	}
+}
+
+static void
+exchange_component_update_accounts (ExchangeComponent *component,
+					gboolean status)
+{
+	ExchangeComponentPrivate *priv = component->priv;
+	ExchangeComponentAccount *baccount;
+	GSList *acc;
+
+	for (acc = priv->accounts; acc; acc = acc->next) {
+		baccount = acc->data;	
+		if (status)
+			exchange_account_set_online (baccount->account);
+		else
+			exchange_account_set_offline (baccount->account);
+	}
+}
+
 static void
 new_connection (MailStubListener *listener, int cmd_fd, int status_fd,
 		ExchangeComponentAccount *baccount)
@@ -258,12 +326,16 @@ new_connection (MailStubListener *listener, int cmd_fd, int status_fd,
 		goto end;
 	}
 
-	if (exchange_account_connect (account))
+	mse = mail_stub_exchange_new (account, cmd_fd, status_fd);
+	/* FIXME : We need to close these sockets */
+/*
+	if (exchange_account_connect (account, NULL, &result))
 		mse = mail_stub_exchange_new (account, cmd_fd, status_fd);
 	else {
 		close (cmd_fd);
 		close (status_fd);
 	}
+*/
 end:
 	g_object_unref (account);
 }
@@ -315,41 +387,18 @@ config_listener_account_removed (ExchangeConfigListener *config_listener,
 	}
 }
 
-
-static struct {
-	char *type, *display_name, *icon;
-} folder_types[] = {
-	{ "mail", N_("Mail"), "stock_folder" },
-	{ "mail/public", NULL, "stock_folder" },
-
-	{ "contacts", N_("Contacts"), "stock_contact" },
-	{ "contacts/public", NULL, "stock_contact" },
-	{ "contacts/ldap", NULL, "stock_contact" },
-
-	{ "tasks", N_("Tasks"), "stock_todo" },
-	{ "tasks/public", NULL, "stock_todo" },
-
-	{ "calendar", N_("Calendar"), "stock_calendar" },
-	{ "calendar/public", NULL, "stock_calendar" }
-};
-static int n_folder_types = G_N_ELEMENTS (folder_types);
-
 static void
-setup_folder_type_registry (ExchangeComponent *component)
+default_linestatus_notify_handler (ExchangeComponent *component, uint status)
 {
-	ExchangeComponentPrivate *priv = component->priv;
-	int i;
+	CORBA_Environment ev;
 
-	priv->folder_type_registry = e_folder_type_registry_new ();
-	for (i = 0; i < n_folder_types; i++) {
-		e_folder_type_registry_register_type (priv->folder_type_registry,
-						      folder_types[i].type,
-						      folder_types[i].icon,
-						      _(folder_types[i].display_name),
-						      NULL,
-						      folder_types[i].display_name != NULL,
-						      0, NULL);
-	}
+	CORBA_exception_init (&ev);
+	GNOME_Evolution_Listener_complete (component->priv->evo_listener, &ev);
+	CORBA_exception_free (&ev);
+	CORBA_Object_release (component->priv->evo_listener, &ev);
+	CORBA_exception_free (&ev);
+
+	component->priv->evo_listener = NULL;
 }
 
 static void
@@ -368,7 +417,21 @@ exchange_component_class_init (ExchangeComponentClass *klass)
 	epv->requestQuit        = impl_requestQuit;
 	epv->quit               = impl_quit;
 	epv->interactive        = impl_interactive;
+	epv->setLineStatus	= impl_setLineStatus;
 
+	klass->linestatus_notify = default_linestatus_notify_handler;
+
+	linestatus_signal_id = 
+		g_signal_new ("linestatus-changed",
+			       G_TYPE_FROM_CLASS (klass),
+			       G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_NO_HOOKS,
+			       G_STRUCT_OFFSET (ExchangeComponentClass, linestatus_notify),
+			       NULL,
+			       NULL,
+			       e_shell_marshal_VOID__INT,
+			       G_TYPE_NONE,
+			       1,
+			       G_TYPE_INT);
 }
 
 static void
@@ -378,8 +441,6 @@ exchange_component_init (ExchangeComponent *component)
 
 	priv = component->priv = g_new0 (ExchangeComponentPrivate, 1);
 
-	setup_folder_type_registry (component);
-
        	priv->config_listener = exchange_config_listener_new ();
 	g_signal_connect (priv->config_listener, "exchange_account_created",
 			  G_CALLBACK (config_listener_account_created),
@@ -387,30 +448,16 @@ exchange_component_init (ExchangeComponent *component)
 	g_signal_connect (priv->config_listener, "exchange_account_removed",
 			  G_CALLBACK (config_listener_account_removed),
 			  component);
-
 }
 
 BONOBO_TYPE_FUNC_FULL (ExchangeComponent, GNOME_Evolution_Component, PARENT_TYPE, exchange_component)
 
-void
-exchange_component_set_offline_listener (ExchangeComponent *component, 
-					 ExchangeOfflineListener *listener)
-{
-	ExchangeComponentPrivate *priv;
-
-	g_return_if_fail (EXCHANGE_IS_COMPONENT (component));
-
-	priv = component->priv;
-	priv->offline_listener = listener;
-}
-
 ExchangeComponent *
 exchange_component_new (void)
 {
-	return g_object_new (EXCHANGE_TYPE_COMPONENT, NULL);
+	return EXCHANGE_COMPONENT (g_object_new (EXCHANGE_TYPE_COMPONENT, NULL));
 }
 
-
 ExchangeAccount *
 exchange_component_get_account_for_uri (ExchangeComponent *component,
 					const char *uri)
@@ -426,8 +473,13 @@ exchange_component_get_account_for_uri (ExchangeComponent *component,
 		if (!uri)
 			return baccount->account;
 
-		if (exchange_account_get_folder (baccount->account, uri))
+		if (exchange_account_get_folder (baccount->account, uri)) {
 			return baccount->account;
+		} else {
+			exchange_account_rescan_tree (baccount->account);
+			if (exchange_account_get_folder (baccount->account, uri))
+				return baccount->account;
+		}
 		/* FIXME : Handle multiple accounts */
 	}
 	return NULL;
@@ -438,7 +490,7 @@ exchange_component_is_offline (ExchangeComponent *component, int *state)
 {
 	g_return_if_fail (EXCHANGE_IS_COMPONENT (component));
 
-	exchange_is_offline (component->priv->offline_listener, state);
+	*state = component->priv->linestatus ? ONLINE_MODE : OFFLINE_MODE;
 }
 
 gboolean
@@ -446,3 +498,17 @@ exchange_component_is_interactive (ExchangeComponent *component)
 {
 	return component->priv->xid != 0;
 }
+
+void
+exchange_component_set_factories (ExchangeComponent *component,
+				  EDataCalFactory *cal_factory,
+				  EDataBookFactory *book_factory)
+{
+	g_return_if_fail (EXCHANGE_IS_COMPONENT (component));
+	g_return_if_fail (E_IS_DATA_CAL_FACTORY (cal_factory));
+	g_return_if_fail (E_IS_DATA_BOOK_FACTORY (book_factory));
+
+	component->priv->cal_factory = g_object_ref (cal_factory);
+	component->priv->book_factory = g_object_ref (book_factory);
+}
+
